/*
 * Copyright 2010-2012 Roman Yepishev <roman.yepishev@ubuntu.com>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3, as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranties of
 * MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

using GLib;

namespace Util {

    const string OLD_EXEC_PATH="/usr/bin/ubuntuone-indicator";
    const string OLD_DESKTOP_FILE="ubuntuone-indicator.desktop";

    public string format_time(int64 seconds) {
        GLib.Time t = GLib.Time();
        int hour = (int)(seconds / 3600);

        if (hour > 24) { 
            return "∞";
        }
        seconds -= hour * 3600; 

        int minute = (int)(seconds / 60);
        seconds -= minute * 60;

        t.hour = hour; 
        t.minute = minute; 
        t.second = (int)seconds;
        t.mktime();
        return t.format("%T");
    }

    public string format_transfer_rate(int64 val) {
        string[] units = {
            // Binary suffixes for data transfer rate
            // Kibibytes per second (not kilobits)
            _("B/s"), _("KiB/s"), _("MiB/s"), _("GiB/s")
        };

        // Transfer rate format: %1$.0f - value, %2$s - unit
        // Leave this as is if "100 MiB/s" is correct in your language
        return _format(val, units, 1024, _("%1$3.0f %2$s"));
    }

    public static string format_data(int64 val) {
        string[] units = {
            // Binary suffixes for data volume (kibibytes, mebibytes etc.)
            _("bytes"), _("KiB"), _("MiB"), _("GiB"), _("TiB")
        };

        // Data volume format: %1$.1f - value, %2$s - unit
        // Leave this as is if "100 MiB" is correct in your language
        return _format(val, units, 1024, _("%1$3.1f %2$s"));
    }

    private static string _format(int64 val, string[] units,
                                  int64 divider, string format) {
        double approx_value = (double)val;
        string current_unit = units[0];

        int units_length = units.length;
        int unit_id = 0;

        foreach (var unit in units) {
            unit_id += 1;
            current_unit = unit;
            if (approx_value >= divider && unit_id != units_length) {
                approx_value /= divider;
            }
            else {
                break;
            }
        }
        
        return format.printf(approx_value, current_unit);
    }

    public void remove_old_autostart_file() {
        string config_dir = Environment.get_user_config_dir();
        string autostart_file = Path.build_filename(config_dir, 
                                    "autostart", OLD_DESKTOP_FILE);

        if (! FileUtils.test(autostart_file, FileTest.EXISTS)) {
            // not removing anything
            return;
        }

        try {
            var kf = new GLib.KeyFile();
            kf.load_from_file(autostart_file, KeyFileFlags.NONE);
            var exec_value = kf.get_string("Desktop Entry", "Exec");

            if (exec_value == OLD_EXEC_PATH) {
                FileUtils.unlink(autostart_file);
            }
        }
        catch(Error e) {
            // nop
        }
    }

    // This is supposed to be using g_utf8_offset which is not in glib vapi
    string utf8_substring(string str, long offset, long len=-1) {
        StringBuilder sb = new StringBuilder.sized(str.char_count());
        unichar c;
        int char_count = 0;
        for (int i=0; str.get_next_char(ref i, out c); char_count++) {
            if (len != -1 && char_count >= len + offset) {
                break;
            }
            if (char_count >= offset) {
                sb.append_unichar(c);
            }
        }
        return sb.str;
    }

}

